@@ -1,33 +1,19 @@
 #include "config.h"
+
 #include <X11/Xlib.h>
-#include <X11/extensions/XShm.h>
-#include <X11/Xutil.h>
-#include <X11/extensions/shape.h>
-#include <X11/Xatom.h>
-#include <X11/Xos.h>
+#include <X11/keysym.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <unistd.h>
+#include <time.h>
+#include <sys/time.h>
 #include <math.h>
 #include <locale.h>
 
-/*
-#include <sys/time.h>
-#include "common.h"
-#include "image.h"
-#include "rend.h"
-#include "rgba.h"
-#include "ximage.h"
-#include "color.h"
- */
 #include "Imlib2.h"
 
 Display            *disp;
 Window              win;
-Visual             *vis;
-Colormap            cm;
-int                 depth;
 
 void                progress(Imlib_Image * im, char percent, int update_x,
                              int update_y, int update_w, int update_h);
@@ -37,8 +23,6 @@ progress(Imlib_Image * im, char percent,
          int update_x, int update_y, int update_w, int update_h)
 {
    imlib_context_set_display(disp);
-   imlib_context_set_visual(vis);
-   imlib_context_set_colormap(cm);
    imlib_context_set_drawable(win);
    imlib_context_set_dither(0);
    imlib_context_set_blend(0);
@@ -278,9 +262,6 @@ main(int argc, char **argv)
              fprintf(stderr, "Can't open display %s\n", display_name);
              return 1;
           }
-        vis = DefaultVisual(disp, DefaultScreen(disp));
-        depth = DefaultDepth(disp, DefaultScreen(disp));
-        cm = DefaultColormap(disp, DefaultScreen(disp));
 #if 0
         /* nasty - using imlib internal function.. but it makes benchmarks fair */
         if (!interactive)
@@ -293,7 +274,7 @@ main(int argc, char **argv)
              win =
                 XCreateSimpleWindow(disp, DefaultRootWindow(disp), 0, 0, 10,
                                     10, 0, 0, 0);
-             XSelectInput(disp, win,
+             XSelectInput(disp, win, KeyPressMask |
                           ButtonPressMask | ButtonReleaseMask | ButtonMotionMask
                           | PointerMotionMask | ExposureMask);
           }
@@ -345,8 +326,8 @@ main(int argc, char **argv)
    if (!blendtest)
      {
         imlib_context_set_display(disp);
-        imlib_context_set_visual(vis);
-        imlib_context_set_colormap(cm);
+        imlib_context_set_visual(DefaultVisual(disp, DefaultScreen(disp)));
+        imlib_context_set_colormap(DefaultColormap(disp, DefaultScreen(disp)));
         imlib_context_set_drawable(win);
      }
    imlib_context_set_anti_alias(aa);
@@ -795,6 +776,7 @@ main(int argc, char **argv)
         Imlib_Updates       up = NULL;
         int                 x, y;
         XEvent              ev;
+        KeySym              keysym;
         Imlib_Font          fn = NULL;
         struct font_hdr {
            int                 type;
@@ -963,14 +945,18 @@ main(int argc, char **argv)
                                                      ev.xexpose.width,
                                                      ev.xexpose.height);
                        break;
+                    case KeyPress:
+                       keysym = XLookupKeysym(&ev.xkey, 0);
+                       if (keysym == XK_q || keysym == XK_Escape)
+                          goto quit;
+                       break;
                     case ButtonRelease:
                        if (fon)
                          {
                             imlib_context_set_font(fn);
                             imlib_free_font();
                          }
-                       exit(0);
-                       break;
+                       goto quit;
                     case MotionNotify:
                        x = ev.xmotion.x;
                        y = ev.xmotion.y;
@@ -1400,5 +1386,16 @@ main(int argc, char **argv)
    printf("%3.3f sec, %3.3f M pixels (%i)\n", sec, (double)pixels / 1000000,
           pixels);
    printf("%3.3f Mpixels / sec\n", (double)(pixels) / (sec * 1000000));
+
+ quit:
+   if (im)
+     {
+        imlib_context_set_image(im);
+        imlib_free_image();
+     }
+   imlib_polygon_free(poly);
+   imlib_polygon_free(poly2);
+   imlib_polygon_free(poly3);
+
    return 0;
 }
