@@ -20,6 +20,8 @@ int                 __imlib_IsRealFile(const char *s);
 
 int                 __imlib_ItemInList(char **list, int size, char *item);
 
-char              **__imlib_ListModules(const char *what, int *num_ret);
+const char         *__imlib_PathToFilters(void);
+const char         *__imlib_PathToLoaders(void);
+char              **__imlib_ListModules(const char *path, int *num_ret);
 
 #endif
