@@ -13,10 +13,11 @@
 static char         _x_err = 0;
 static DATA8        rtab[256], gtab[256], btab[256];
 
-static void
+static int
 Tmp_HandleXError(Display * d, XErrorEvent * ev)
 {
    _x_err = 1;
+   return 0;
 }
 
 void
@@ -25,7 +26,8 @@ __imlib_GrabXImageToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
                          int depth, int x, int y, int w, int h, char grab)
 {
    int                 inx, iny;
-   DATA32             *src, *ptr;
+   const DATA32       *src;
+   DATA32             *ptr;
    int                 pixel;
    int                 origx, origy;
    int                 bgr = 0;
@@ -53,14 +55,6 @@ __imlib_GrabXImageToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
    if ((depth == 24) && (xim->bits_per_pixel == 32))
       depth = 25;               /* fake depth meaning 24 bit in 32 bpp ximage */
    /* data needs swapping */
-#define SWAP32(x) (x) = \
-   ((((int)(x) & 0x000000ff ) << 24) |\
-       (((int)(x) & 0x0000ff00 ) << 8) |\
-       (((int)(x) & 0x00ff0000 ) >> 8) |\
-       (((int)(x) & 0xff000000 ) >> 24))
-#define SWAP16(x) (x) = \
-   ((((short)(x) & 0x00ff ) << 8) |\
-       (((short)(x) & 0xff00 ) >> 8))
 
 #ifdef WORDS_BIGENDIAN
    if (xim->bitmap_bit_order == LSBFirst)
@@ -90,7 +84,7 @@ __imlib_GrabXImageToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
                      (unsigned short *)(xim->data + (xim->bytes_per_line * y));
                   for (x = 0; x < w; x++)
                     {
-                       SWAP16(*tmp);
+                       *tmp = SWAP16(*tmp);
                        tmp++;
                     }
                }
@@ -105,7 +99,7 @@ __imlib_GrabXImageToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
                   tmp = (unsigned int *)(xim->data + (xim->bytes_per_line * y));
                   for (x = 0; x < w; x++)
                     {
-                       SWAP32(*tmp);
+                       *tmp = SWAP32(*tmp);
                        tmp++;
                     }
                }
@@ -540,7 +534,7 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
    if (grab)
       XGrabServer(d);
    XSync(d, False);
-   prev_erh = XSetErrorHandler((XErrorHandler) Tmp_HandleXError);
+   prev_erh = XSetErrorHandler(Tmp_HandleXError);
    _x_err = 0;
    /* lets see if its a pixmap or not */
    XGetWindowAttributes(d, p, &xatt);
@@ -659,15 +653,9 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
      }
 
    /* Create an Ximage (shared or not) */
-   if (x_does_shm < 0)
-      __imlib_ShmCheck(d);
+   xim = __imlib_ShmGetXImage(d, v, p, xatt.depth, x, y, w, h, &shminfo);
+   is_shm = !!xim;
 
-   xim = NULL;
-   if (x_does_shm)
-     {
-        xim = __imlib_ShmGetXImage(d, v, p, xatt.depth, x, y, w, h, &shminfo);
-        is_shm = ! !xim;
-     }
    if (!xim)
       xim = XGetImage(d, p, x, y, w, h, 0xffffffff, ZPixmap);
    if (!xim)
@@ -681,7 +669,7 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
    if ((m) && (domask))
      {
         mxim = __imlib_ShmGetXImage(d, v, m, 1, 0, 0, w, h, &mshminfo);
-        is_mshm = ! !mxim;
+        is_mshm = !!mxim;
         if (!mxim)
            mxim = XGetImage(d, m, 0, 0, w, h, 0xffffffff, ZPixmap);
      }
@@ -739,13 +727,16 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
 
    /* destroy the Ximage */
    if (is_shm)
-      __imlib_ShmDetach(d, &shminfo);
-   XDestroyImage(xim);
+      __imlib_ShmDestroyXImage(d, xim, &shminfo);
+   else
+      XDestroyImage(xim);
+
    if (mxim)
      {
         if (is_mshm)
-           __imlib_ShmDetach(d, &mshminfo);
-        XDestroyImage(mxim);
+           __imlib_ShmDestroyXImage(d, mxim, &mshminfo);
+        else
+           XDestroyImage(mxim);
      }
    if (created_mask)
       XFreePixmap(d, m);
