@@ -6,20 +6,20 @@
 
 #include "common.h"
 
-extern signed char  x_does_shm;
-
-void                __imlib_SetMaxXImageCount(Display * d, int num);
-int                 __imlib_GetMaxXImageCount(Display * d);
-void                __imlib_SetMaxXImageTotalSize(Display * d, int num);
-int                 __imlib_GetMaxXImageTotalSize(Display * d);
+void                __imlib_SetXImageCacheCountMax(Display * d, int num);
+int                 __imlib_GetXImageCacheCountMax(Display * d);
+int                 __imlib_GetXImageCacheCountUsed(Display * d);
+void                __imlib_SetXImageCacheSizeMax(Display * d, int num);
+int                 __imlib_GetXImageCacheSizeMax(Display * d);
+int                 __imlib_GetXImageCacheSizeUsed(Display * d);
 void                __imlib_FlushXImage(Display * d);
 void                __imlib_ConsumeXImage(Display * d, XImage * xim);
 XImage             *__imlib_ProduceXImage(Display * d, Visual * v, int depth,
                                           int w, int h, char *shared);
-void                __imlib_ShmCheck(Display * d);
 XImage             *__imlib_ShmGetXImage(Display * d, Visual * v, Drawable draw,
                                          int depth, int x, int y, int w, int h,
                                          XShmSegmentInfo * si);
-void                __imlib_ShmDetach(Display * d, XShmSegmentInfo * si);
+void                __imlib_ShmDestroyXImage(Display * d, XImage * xim,
+                                             XShmSegmentInfo * si);
 
 #endif
